/**
 * DevExtreme (cjs/__internal/grids/tree_list/editing/m_editing.js)
 * Version: 23.1.11 (build 24189-0109)
 * Build date: Sun Jul 07 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
require("../module_not_extended/editor_factory");
var _renderer = _interopRequireDefault(require("../../../../core/renderer"));
var _deferred = require("../../../../core/utils/deferred");
var _extend = require("../../../../core/utils/extend");
var _type = require("../../../../core/utils/type");
var _message = _interopRequireDefault(require("../../../../localization/message"));
var _ui = _interopRequireDefault(require("../../../../ui/widget/ui.errors"));
var _m_editing = require("../../../grids/grid_core/editing/m_editing");
var _m_utils = _interopRequireDefault(require("../../../grids/grid_core/m_utils"));
var _m_core = _interopRequireDefault(require("../m_core"));

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}

function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf(subClass, superClass)
}

function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function(o, p) {
        o.__proto__ = p;
        return o
    };
    return _setPrototypeOf(o, p)
}
var TREELIST_EXPAND_ICON_CONTAINER_CLASS = "dx-treelist-icon-container";
var SELECT_CHECKBOX_CLASS = "dx-select-checkbox";
var DATA_EDIT_DATA_INSERT_TYPE = "insert";
var EditingController = function(_editingModule$contro) {
    _inheritsLoose(EditingController, _editingModule$contro);

    function EditingController() {
        return _editingModule$contro.apply(this, arguments) || this
    }
    var _proto = EditingController.prototype;
    _proto._generateNewItem = function(key) {
        var item = _editingModule$contro.prototype._generateNewItem.call(this, key);
        item.data = {
            key: key
        };
        item.children = [];
        item.level = 0;
        item.parentKey = this.option("rootValue");
        return item
    };
    _proto._isProcessedItem = function() {
        return true
    };
    _proto._setInsertAfterOrBeforeKey = function(change, parentKey) {
        if (void 0 !== parentKey && parentKey !== this.option("rootValue")) {
            change.insertAfterKey = parentKey
        } else {
            _editingModule$contro.prototype._setInsertAfterOrBeforeKey.apply(this, arguments)
        }
    };
    _proto._getLoadedRowIndex = function(items, change) {
        var dataSourceAdapter = this._dataController.dataSource();
        var parentKey = null === dataSourceAdapter || void 0 === dataSourceAdapter ? void 0 : dataSourceAdapter.parentKeyOf(change.data);
        if (void 0 !== parentKey && parentKey !== this.option("rootValue")) {
            var rowIndex = _m_utils.default.getIndexByKey(parentKey, items);
            if (rowIndex >= 0 && this._dataController.isRowExpanded(parentKey)) {
                return rowIndex + 1
            }
            return -1
        }
        return _editingModule$contro.prototype._getLoadedRowIndex.apply(this, arguments)
    };
    _proto._isEditColumnVisible = function() {
        var result = _editingModule$contro.prototype._isEditColumnVisible.apply(this, arguments);
        var editingOptions = this.option("editing");
        return result || editingOptions.allowAdding
    };
    _proto._isDefaultButtonVisible = function(button, options) {
        var result = _editingModule$contro.prototype._isDefaultButtonVisible.apply(this, arguments);
        var row = options.row;
        if ("add" === button.name) {
            return this.allowAdding(options) && row.rowIndex !== this._getVisibleEditRowIndex() && !(row.removed || row.isNewRow)
        }
        return result
    };
    _proto._getEditingButtons = function(options) {
        var buttons = _editingModule$contro.prototype._getEditingButtons.apply(this, arguments);
        if (!options.column.buttons) {
            buttons.unshift(this._getButtonConfig("add", options))
        }
        return buttons
    };
    _proto._beforeSaveEditData = function(change) {
        var _a;
        var result = _editingModule$contro.prototype._beforeSaveEditData.apply(this, arguments);
        if (change && change.type !== DATA_EDIT_DATA_INSERT_TYPE) {
            var store = null === (_a = this._dataController) || void 0 === _a ? void 0 : _a.store();
            var key = null === store || void 0 === store ? void 0 : store.key();
            if (!(0, _type.isDefined)(key)) {
                throw _ui.default.Error("E1045")
            }
        }
        return result
    };
    _proto.addRowByRowIndex = function(rowIndex) {
        var row = this._dataController.getVisibleRows()[rowIndex];
        return this.addRow(row ? row.key : void 0)
    };
    _proto.addRow = function(key) {
        if (void 0 === key) {
            key = this.option("rootValue")
        }
        return _editingModule$contro.prototype.addRow.call(this, key)
    };
    _proto._addRowCore = function(data, parentKey, oldEditRowIndex) {
        var _this = this;
        var rootValue = this.option("rootValue");
        var dataSourceAdapter = this._dataController.dataSource();
        var parentKeyGetter = dataSourceAdapter.createParentIdGetter();
        parentKey = parentKeyGetter(data);
        if (void 0 !== parentKey && parentKey !== rootValue && !this._dataController.isRowExpanded(parentKey)) {
            var deferred = new _deferred.Deferred;
            this._dataController.expandRow(parentKey).done((function() {
                setTimeout((function() {
                    _editingModule$contro.prototype._addRowCore.call(_this, data, parentKey, oldEditRowIndex).done(deferred.resolve).fail(deferred.reject)
                }))
            })).fail(deferred.reject);
            return deferred.promise()
        }
        return _editingModule$contro.prototype._addRowCore.call(this, data, parentKey, oldEditRowIndex)
    };
    _proto._initNewRow = function(options, parentKey) {
        var dataSourceAdapter = this._dataController.dataSource();
        var parentIdSetter = dataSourceAdapter.createParentIdSetter();
        parentIdSetter(options.data, parentKey);
        return _editingModule$contro.prototype._initNewRow.apply(this, arguments)
    };
    _proto.allowAdding = function(options) {
        return this._allowEditAction("allowAdding", options)
    };
    _proto._needToCloseEditableCell = function($targetElement) {
        return _editingModule$contro.prototype._needToCloseEditableCell.apply(this, arguments) || $targetElement.closest(".".concat(TREELIST_EXPAND_ICON_CONTAINER_CLASS)).length && this.isEditing()
    };
    _proto.getButtonLocalizationNames = function() {
        var names = _editingModule$contro.prototype.getButtonLocalizationNames.apply(this);
        names.add = "dxTreeList-editingAddRowToNode";
        return names
    };
    return EditingController
}(_m_editing.editingModule.controllers.editing);
var rowsView = function(Base) {
    return function(_editingModule$extend) {
        _inheritsLoose(TreeListEditingRowsViewExtender, _editingModule$extend);

        function TreeListEditingRowsViewExtender() {
            return _editingModule$extend.apply(this, arguments) || this
        }
        var _proto2 = TreeListEditingRowsViewExtender.prototype;
        _proto2._renderCellCommandContent = function($container, options) {
            var editingController = this._editingController;
            var isEditRow = options.row && editingController.isEditRow(options.row.rowIndex);
            var isEditing = options.isEditing || isEditRow;
            if (!options.isOnForm && !isEditing) {
                return _editingModule$extend.prototype._renderCellCommandContent.apply(this, arguments)
            }
            return false
        };
        _proto2.validateClick = function(e) {
            var $targetElement = (0, _renderer.default)(e.event.target);
            var originalClickHandler = "dxdblclick" === e.event.type ? _editingModule$extend.prototype._rowDblClick : _editingModule$extend.prototype._rowClick;
            if ($targetElement.closest(".".concat(SELECT_CHECKBOX_CLASS)).length) {
                return false
            }
            return !this.needToCallOriginalClickHandler(e, originalClickHandler)
        };
        _proto2.needToCallOriginalClickHandler = function(e, originalClickHandler) {
            var $targetElement = (0, _renderer.default)(e.event.target);
            if (!$targetElement.closest(".".concat(TREELIST_EXPAND_ICON_CONTAINER_CLASS)).length) {
                originalClickHandler.call(this, e);
                return true
            }
            return false
        };
        _proto2._rowClick = function(e) {
            if (this.validateClick(e)) {
                _editingModule$extend.prototype._rowClickTreeListHack.apply(this, arguments)
            }
        };
        _proto2._rowDblClick = function(e) {
            if (this.validateClick(e)) {
                _editingModule$extend.prototype._rowDblClickTreeListHack.apply(this, arguments)
            }
        };
        return TreeListEditingRowsViewExtender
    }(_m_editing.editingModule.extenders.views.rowsView(Base))
};
var data = function(Base) {
    return function(_dataControllerEditin) {
        _inheritsLoose(DataControllerTreeListEditingExtender, _dataControllerEditin);

        function DataControllerTreeListEditingExtender() {
            return _dataControllerEditin.apply(this, arguments) || this
        }
        var _proto3 = DataControllerTreeListEditingExtender.prototype;
        _proto3.changeRowExpand = function() {
            this._editingController.refresh();
            return _dataControllerEditin.prototype.changeRowExpand.apply(this, arguments)
        };
        return DataControllerTreeListEditingExtender
    }((0, _m_editing.dataControllerEditingExtenderMixin)(Base))
};
_m_core.default.registerModule("editing", {
    defaultOptions: function() {
        return (0, _extend.extend)(true, _m_editing.editingModule.defaultOptions(), {
            editing: {
                texts: {
                    addRowToNode: _message.default.format("dxTreeList-editingAddRowToNode")
                }
            }
        })
    },
    controllers: {
        editing: EditingController
    },
    extenders: {
        controllers: {
            data: data
        },
        views: {
            rowsView: rowsView,
            headerPanel: _m_editing.editingModule.extenders.views.headerPanel
        }
    }
});
